@@ -15,14 +15,21 @@
 
 import threading,time,logging,uuid,weakref,gc,uuid,traceback,os
 
-from . import workers
+from . import workers,messagebus
 
 initialized = False
 initlock = threading.Lock()
+
+lock = threading.RLock()
 Gst = None
 lock = threading.RLock()
 jackChannels = {}
 
+def tryToAvoidSegfaults(t,v):
+    if v.clientName=="system":
+        stopAllJackUsers()
+messagebus.subscribe("/system/jack/delport",tryToAvoidSegfaults)
+
 pipes = weakref.WeakValueDictionary()
 
 log = logging.getLogger("IceFlow_gst")
@@ -48,6 +55,17 @@ class PILCapture():
 
 
 
+class PILSource():
+    def __init__(self,appsrc,greyscale=False):
+        self.appsrc=appsrc
+        self.greyscale = greyscale
+
+
+    def push(self,img):
+        img = img.tobytes("raw","L" if self.greyscale else "rgb")
+        img = Gst.Buffer.new_wrapped(img)
+        self.appsrc.emit("push-buffer", img)
+
 def link(a,b):
     unref = False
     try:
@@ -73,7 +91,9 @@ def stopAllJackUsers():
     with lock:
         c = list(jackChannels.items())
         for i in c:
+            #Sync stop, we gotta wait
             try:
+                i[1]().syncStop=True
                 i[1]().stop()
             except:
                 log.exception("Err stopping JACK user")
@@ -120,10 +140,10 @@ def init():
 
             #mainContext = GLib.MainContext()
 
-            glibloop = GLib.MainLoop()
+            #glibloop = GLib.MainLoop()
 
-            loop=threading.Thread(target=glibloop.run,daemon=True)
-            loop.start()
+            #loop=threading.Thread(target=glibloop.run,daemon=True)
+            #loop.start()
             initialized = True
 
 
@@ -148,12 +168,7 @@ def makeWeakrefPoller(selfref,exitSignal):
 
     def pollerf():
         alreadyStarted = False
-        lastAdjustment=0
-        #Last time we adjusted, how much error was left?
-        lastRemnantError=0
-        computeRemnant=False
-
-        avgDiff = 0
+        t=0
         while selfref():
             self=selfref()
             
@@ -163,32 +178,47 @@ def makeWeakrefPoller(selfref,exitSignal):
                 return
             
             if self.running:
-                t=time.monotonic()
-                try:
-                    with self.lock:
-                        self.loopCallback()
-                    
-                        state = self.pipeline.get_state(1000000000)[1]
-                    if not state==Gst.State.NULL:
-                        self.wasEverRunning=True
+                if t<time.monotonic()-3:
+                    t=time.monotonic()
+                    try:
+                        with self.lock:
+                            self.loopCallback()
+                        
+                            state = self.pipeline.get_state(1000000000)[1]
+                        if not state==Gst.State.NULL:
+                            self.wasEverRunning=True
 
-                    #Set the flag if anything ever drives us into the null state
-                    if self.wasEverRunning and state==Gst.State.NULL:
-                        self.running=False
+                        #Set the flag if anything ever drives us into the null state
+                        if self.wasEverRunning and state==Gst.State.NULL:
+                            self.running=False
+                            exitSignal.append(True)
+                            return
+
+                    except:
+                        #Todo actually handle some errors?
                         exitSignal.append(True)
-                        return
+                        #After pipeline deleted, we clean up
+                        if hasattr(self,'pipeline') and  hasattr(self,'bus'):
+                            raise
+                        else:
+                            return
+            with self.lock:
+                msg = self.bus.timed_pop(100*1000)
+                if msg:
+                    try:
+                        if msg.type == Gst.MessageType.ERROR:
+                            self.on_error(self.bus,msg,None)
+                        if msg.type == Gst.MessageType.EOS:
+                            self.on_eos(self.bus,msg,None)
+                        self.on_message(self.bus,msg,None)
+                    except:
+                        logging.exception("Err in pipeline:"+self.name)
+                    finally:
+                        pass
 
-                except:
-                    #Todo actually handle some errors?
-                    exitSignal.append(True)
-                    #After pipeline deleted, we clean up
-                    if hasattr(self,'pipeline') and  hasattr(self,'bus'):
-                        raise
-                    else:
-                        return
-            
             del self
-            time.sleep(1)
+
+            #time.sleep(1)
     return pollerf
 
 
@@ -228,11 +258,13 @@ class GStreamerPipeline():
         self.realtime = realtime
         self.lock = threading.RLock()
         self._stopped=True
+        self.syncStop=False
         self.pipeline = Gst.Pipeline()
         self.threadStarted=False
         self.weakrefs = weakref.WeakValueDictionary()
 
 
+
         #This WeakValueDictionary is mostly for testing purposes
         pipes[id(self)]=self
         
@@ -274,6 +306,7 @@ class GStreamerPipeline():
         self.name = name
 
         self.elements = []
+        self.sidechainElements = []
         self.namedElements = {}
 
         self.elementTypesById= {}
@@ -289,8 +322,10 @@ class GStreamerPipeline():
         self.startTime = 0
         def dummy(*a,**k):
             pass
-        self._syncmessage = wrfunc(weakref.WeakMethod(self.syncMessage),fail_return=Gst.BusSyncReply.PASS)
-        self.bus.set_sync_handler(self._syncmessage,0,dummy)
+            
+        if realtime:
+            self._syncmessage = wrfunc(weakref.WeakMethod(self.syncMessage),fail_return=Gst.BusSyncReply.PASS)
+            self.bus.set_sync_handler(self._syncmessage,0,dummy)
         self.pollthread=None
 
         self.lastElementType = None
@@ -301,7 +336,10 @@ class GStreamerPipeline():
         self.pipelineRate = 1.0
 
 
-        
+
+    def sendEOS(self):
+        self.pipeline.send_event(Gst.Event.new_eos())    
+
     def loopCallback(self):
         #Meant to subclass. Gets called under the lock
         pass
@@ -436,51 +474,52 @@ class GStreamerPipeline():
         t=time.monotonic()
         while not self.pipeline.get_state(1000_000_000)[1]==s:
             if time.monotonic()-t> timeout:
-                raise RuntimeError("Timeout")
+                raise RuntimeError("Timeout, pipeline still in: ", self.pipeline.get_state(1000_000_000)[1])
             time.sleep(0.1)
 
-    def start(self, effectiveStartTime=None):
+    def start(self, effectiveStartTime=None,timeout=10):
         "effectiveStartTime is used to keep multiple players synced when used with systemTime"
-        with self.lock:
-            if self.exiting:
-                return
+        with lock:
+            with self.lock:
+                if self.exiting:
+                    return
 
-                        
-            x = effectiveStartTime or time.time()
-            timeAgo = time.time()-x
-            #Convert to monotonic time that the nternal APIs use
-            self.startTime= time.monotonic()-timeAgo
-            
-            
-            if not self.pipeline.get_state(1000_000_000)[1] ==Gst.State.PAUSED:
-                self.pipeline.set_state(Gst.State.PAUSED)
-                self._waitForState(Gst.State.PAUSED)
-            
-            #Seek to where we should be, if we had actually
-            #Started when we should have. We want to get everything set up in the pause state
-            #First so we have the right "effective" start time.
+                            
+                x = effectiveStartTime or time.time()
+                timeAgo = time.time()-x
+                #Convert to monotonic time that the nternal APIs use
+                self.startTime= time.monotonic()-timeAgo
+                
+                
+                if not self.pipeline.get_state(1000_000_000)[1] ==Gst.State.PAUSED:
+                    self.pipeline.set_state(Gst.State.PAUSED)
+                    self._waitForState(Gst.State.PAUSED)
+                
+                #Seek to where we should be, if we had actually
+                #Started when we should have. We want to get everything set up in the pause state
+                #First so we have the right "effective" start time.
 
-            #We accept cutting off a few 100 milliseconds if it means
-            #staying synced.
-            if self.systemTime:
-                self.seek(time.monotonic()-self.startTime)
+                #We accept cutting off a few 100 milliseconds if it means
+                #staying synced.
+                if self.systemTime:
+                    self.seek(time.monotonic()-self.startTime)
 
-            self.pipeline.set_state(Gst.State.PLAYING)
-            self._waitForState(Gst.State.PLAYING)
-            self.running=True
+                self.pipeline.set_state(Gst.State.PLAYING)
+                self._waitForState(Gst.State.PLAYING,timeout)
+                self.running=True
 
-            for i in range(0,50):
-                try:
-                    #Test that we can actually read the clock
-                    self.getPosition()
-                    break
-                except:
-                    if i>48:
-                        raise RuntimeError("Clock still not valid")
-                    time.sleep(0.1)
+                for i in range(0,500):
+                    try:
+                        #Test that we can actually read the clock
+                        self.getPosition()
+                        break
+                    except:
+                        if i>150:
+                            raise RuntimeError("Clock still not valid")
+                        time.sleep(0.1)
 
-            #Don't start the thread until we have a valid clock
-            self.maybeStartPoller()
+                #Don't start the thread until we have a valid clock
+                self.maybeStartPoller()
 
     def play(self):
         with self.lock:
@@ -516,19 +555,23 @@ class GStreamerPipeline():
             self.pollthread.start()
 
     def stop(self):
-        #Actually stop as soon as we can
-        with self.lock:
-            #This was causing segfaults for some reasons
-            if not (self.pipeline.get_state(1000_000_000)[1]==Gst.State.NULL):
-                self.pipeline.set_state(Gst.State.NULL)
-            self.exiting = True
-            self.bus.set_sync_handler(None,0,None)
-            if self.hasSignalWatch:
-                self.bus.remove_signal_watch()
-                self.hasSignalWatch = False
+        with lock:
+            #Actually stop as soon as we can
+            with self.lock:
+                if hasattr(self,'pipeline'):
+                    #This was causing segfaults for some reasons
+                    if not (self.pipeline.get_state(1000_000_000)[1]==Gst.State.NULL):
+                        self.pipeline.set_state(Gst.State.NULL)
+                self.exiting = True
+                if hasattr(self,'bus'):
+                    self.bus.set_sync_handler(None,0,None)
+                    if self.hasSignalWatch:
+                        self.bus.remove_signal_watch()
+                        self.hasSignalWatch = False
         
         #Now we're going to do the cleanup stuff
-        #In the background, because it involves a lot of waiting
+        #In the background, because it involves a lot of waiting.
+        #This might fail, if it never even started, but we just kinda ignore that.
         def gstStopCleanupTask():
             self.running=False
             t = time.monotonic()
@@ -582,7 +625,11 @@ class GStreamerPipeline():
                     pass
                 self._stopped=True
 
-        workers.do(gstStopCleanupTask)
+        #Allow waiting so we can be real sure it's stopped
+        if not self.syncStop:
+            workers.do(gstStopCleanupTask)
+        else:
+            gstStopCleanupTask()
         
        
 
@@ -598,67 +645,108 @@ class GStreamerPipeline():
         appsink = self.addElement("appsink",drop=True,sync=False,max_buffers=buffer)
 
         return PILCapture(appsink)
+
+    def addPILSource(self,resolution, buffer=1,greyscale=False):
+        "Return a video source object that we can use to put PIL buffers into the stream"
+
+        appsrc = self.addElement("appsrc",caps="video/x-raw,width="+str(resolution[0])+",height="+str(resolution[0])+", format="+"GREy8" if greyscale else "RGB",connectToOutput=False)
+        conv = self.addElement("videoconvert")
+        scale=self.addElement("videoscale")
+
+        #Start with a blck image to make things prerooll
+        if(greyscale):
+            appsrc.emit("push-buffer", Gst.Buffer.new_wrapped(bytes(resolution[0]*resolution[1])))
+        else:
+            appsrc.emit("push-buffer",Gst.Buffer.new_wrapped( bytes(resolution[0]*resolution[1]*3)))
+
+      
+
+        return PILSource(appsrc,greyscale)
     
-    def addElement(self,t,name=None,connectWhenAvailable=False, connectToOutput=None, **kwargs):
+    def addElement(self,t,name=None,connectWhenAvailable=False, connectToOutput=None, sidechain=False, **kwargs):
 
         #Don't let the user use JACK if it's not running,
         #For fear of gstreamer undefined behavior
         if t.startswith("jackaudio"):
             if not self.shouldAllowGstJack():
                 raise RuntimeError("JACK not running")
-        
-        with self.lock:
-            if not isinstance(t, str):
-                raise ValueError("Element type must be string")
+        if lock.acquire(timeout=10):
+            try:
+                with self.lock:
+                    if not isinstance(t, str):
+                        raise ValueError("Element type must be string")
 
-            e = Gst.ElementFactory.make(t,name)
-            
-            if e==None:
-                raise ValueError("Nonexistant element type: "+t)
-            self.weakrefs[str(e)]=e
-            self.elementTypesById[id(e)] = t
+                    e = Gst.ElementFactory.make(t,name)
+                    
+                    if e==None:
+                        raise ValueError("Nonexistant element type: "+t)
+                    self.weakrefs[str(e)]=e
+                    self.elementTypesById[id(e)] = t
 
 
-            for i in kwargs:
-                v = kwargs[i]
-                self.setProperty(e,i,v)
+                    for i in kwargs:
+                        v = kwargs[i]
+                        self.setProperty(e,i,v)
+                        
+                    self.pipeline.add(e)
+
                 
-            self.pipeline.add(e)
+                    if connectToOutput:
+                        if not id(connectToOutput) in self.elementTypesById:
+                            raise ValueError("Cannot connect to the output of: "+str(connectToOutput)+", no such element in pipeline.")
+                    
+                    #Element doesn't have an input pad, we want this to be usable as a fake source to go after a real source if someone
+                    #wants to use it as a effect
+                    if t=="audiotestsrc":
+                        connectToOutput=False
+                        
 
+
+                    #This could be the first element
+                    if self.elements and (not (connectToOutput is False)):
+                        connectToOutput=connectToOutput or self.elements[-1]
+
+                        #Fakesinks have no output, we automatically don't connect those
+                        if self.elementTypesById[id(connectToOutput)]=='fakesink':
+                            connectToOutput=False
+
+                        #Decodebin doesn't have a pad yet for some awful reason
+                        elif (self.elementTypesById[id(connectToOutput)]=='decodebin') or connectWhenAvailable:
+                            eid = uuid.uuid4()
+                            f = linkClosureMaker(weakref.ref(self),connectToOutput,e,connectWhenAvailable, eid)
+
+                            self.waitingCallbacks[eid]=f
+                            #Dummy 1 param because some have claimed to get segfaults without
+                            connectToOutput.connect("pad-added",f,1)
+                        else:
+                            link(connectToOutput,e)
+                    
+                    #Sidechain means don't set this element as the
+                    #automatic thing that the next entry links to
+                    if not sidechain:
+                        self.elements.append(e)
+                    else:
+                        self.sidechainElements.append(e)
+
+                    self.namedElements[name]=e
+
+                    #Mark as a JACK user so we can stop if needed for JACK
+                    #Stuff
+                    if t.startswith("jackaudio"):
+                        with lock:
+                            jackChannels[self.uuid] = weakref.ref(self)
+
+                    self.lastElementType = t
+                    p= weakref.proxy(e)
+                    #List it under the proxy as well
+                    self.elementTypesById[id(p)] = t
+                    return p
+
+            finally:
+                lock.release()
+        else:
+            raise RuntimeError("Getting lock")
         
-            if connectToOutput:
-                if not id(connectToOutput) in self.elementTypesById:
-                    raise ValueError("Cannot connect to the output of: "+str(connectToOutput)+", no such element in pipeline.")
-
-            #This could be the first element
-            if self.elements:
-                connectToOutput=connectToOutput or self.elements[-1]
-                #Decodebin doesn't have a pad yet for some awful reason
-                if (self.elementTypesById[id(connectToOutput)]=='decodebin') or connectWhenAvailable:
-                    eid = uuid.uuid4()
-                    f = linkClosureMaker(weakref.ref(self),connectToOutput,e,connectWhenAvailable, eid)
-
-                    self.waitingCallbacks[eid]=f
-                    #Dummy 1 param because some have claimed to get segfaults without
-                    connectToOutput.connect("pad-added",f,1)
-                else:
-                    link(connectToOutput,e)
-            self.elements.append(e)
-            self.namedElements[name]=e
-
-            #Mark as a JACK user so we can stop if needed for JACK
-            #Stuff
-            if t.startswith("jackaudio"):
-                with lock:
-                    jackChannels[self.uuid] = weakref.ref(self)
-
-            self.lastElementType = t
-            p= weakref.proxy(e)
-            #List it under the proxy as well
-            self.elementTypesById[id(p)] = t
-            return p
-
-
     def setProperty(self, element, prop,value):
         with self.lock:
 
