@@ -325,7 +325,10 @@ class _TagPoint(virtualresource.VirtualResource):
     def value(self):
         return self._getValue()
 
-    def _getValue(self):
+    def pull(self):
+        return self._getValue(True)
+
+    def _getValue(self,force=False):
         #Get the numeric value of the tag. It is meant to be called under lock.
 
         activeClaimValue = self._value
@@ -335,7 +338,7 @@ class _TagPoint(virtualresource.VirtualResource):
             #Rate limited tag getter logic. We ignore the possibility for
             #Race conditions and assume that calling a little too often is fine, since
             #It shouldn't affect correctness
-            if time.time()-self.lastGotValue> self.interval:
+            if (time.time()-self.lastGotValue> self.interval) or force:
                 #Set this flag immediately, or else a function with an error could defeat the cacheing
                 #And just flood everything with errors
                 self.lastGotValue = time.time()
@@ -444,21 +447,26 @@ class _TagPoint(virtualresource.VirtualResource):
             #If priority has been changed on the existing active claim
             #We need to handle it
             elif name==self.activeClaim[2]:
-                #Defensive programming against weakrefs dissapearing
-                #in some kind of race condition that leaves them in the list.
-                #Basically we find the highest priority valid claim
-                for i in reversed(sorted(self.claims.values())):
-                    x= i[3]()
-                    if x:
-                        self._value=x.value
-                        self.timestamp = x.timestamp
-                        self.annotation = x.annotation
-                        self.activeClaim=i
-                        self.handleSourceChanged(i[2])
-                        break
+                self._findActiveClaim()
+
             self._push(self._getValue(),self.timestamp, self.annotation)           
             return claim
 
+    def _findActiveClaim(self):
+        #Defensive programming against weakrefs dissapearing
+        #in some kind of race condition that leaves them in the list.
+        #Basically we find the highest priority valid claim
+        for i in reversed(sorted(self.claims.values())):
+            x= i[3]()
+            if x:
+                self._value=x.value
+                self.timestamp = x.timestamp
+                self.annotation = x.annotation
+                self.activeClaim=i
+                self.handleSourceChanged(i[2])
+                break
+
+
     def setClaimVal(self,claim,val,timestamp,annotation):
         "Set the value of an existing claim"
         timestamp = timestamp or time.monotonic()
@@ -473,6 +481,12 @@ class _TagPoint(virtualresource.VirtualResource):
                 upd=True
             else:
                 upd=False
+                #If we have the same priority as the active claim, even though we aren't active, by being set,
+                #we can steal the active status, because this is now newer.
+                if c.priority>=self.activeClaim.priority:
+                    self.activeClaim=c
+                    upd=True
+
             #Grab the claim obj and set it's val
             x= c[3]()
             x.value = val
@@ -618,6 +632,10 @@ class _NumericTagPoint(_TagPoint):
                 if value:
                     raise ValueError("Cannot change unit of tagpoint. To override this, set to None first")
         self._unit = value
+        #Maybe race condition here, but most likely nobody is setting this
+        #from separate threads. TODO
+        if not self._displayUnits:
+            self._displayUnits=value
         self.onConfigChanged()
        
 
